from larrt import planning
import pickle as pkl
import json

from mrmh_model import terrain, human, montecarlo, searcher
import plotting_utils as plotter
import waypoint_maker
import matplotlib.pyplot as plt
from scouter.terrain_viewer import plot_all
from matplotlib.backends.backend_pdf import PdfPages
from scipy import ndimage

from mrmh_model import montecarlo as MC
import numpy as np
from mrmh_model import searcher
import torch
from gp import beziergp
from gp import robotgp
import sys
from mrmh_model.params import Default
import datetime as dt
import time, os
import pdb

#Relative filepaths (Assumes this and ags_grabber projects are in same parent directory)
kentland_heatmap = './LP model/analysis/outputs/ic_1_con_hiker_t12_kentland.csv'
hmpark_heatmap = './LP model/analysis/outputs/ic_1_con_hiker_t12_hmpark.csv'

kentland_linfeat = '../ags_grabber/matlab_data/BW_LFandInac_Zelev_kentland.mat'
hmpark_linfeat = '../ags_grabber/matlab_data/BW_LFandInac_Zelev_hmpark.mat'


def create_data():

    #Set up environment parameters
    n_max = 3   #Number of robots
    s_max = 0   #Number of searchers (humans)
    h_max = 0   #Number of humans?
    start_time = time.time()

    #Dictionary of parameters
    params = ({
        'lp_model': 'ring',    #custom, ring, trust, stripes
        'opt_iterations': 1000,
        'path_style': 'basic',
        'stats_name': 'kentland',
        'anchor_point': [37.197730, -80.585233], # kentland
        'num_searchers': s_max,
        'num_robots': n_max,
        'num_humans' : h_max,
        'lp_filename': kentland_heatmap,
        'lin_feat_filename': kentland_linfeat,
        'res': 25,
        'save_data': True
    })
    params['save_folder'] = 'deeprl/{}_n{}_s{}_{}'.format(
        params['stats_name'],n_max,s_max,params['path_style'])
    params = Default(params).params


    #MonteCarlo loads LPM, creates searchers (humans on ground)
    mc = MC.MonteCarlo(params=params) # calls terrain builder
    mc.run_experiment()

    #Put Ring search values and paths into one matrix for the gym Environment (have to fix for full res)
    gridsize_x = mc._x_shape*params['res']
    gridsize_y = mc._y_shape*params['res']
    sar_fullgrid = np.zeros((gridsize_x,gridsize_y),dtype=float)
    [x_crds,y_crds] = np.meshgrid(np.linspace(params['xlims'][0], params['xlims'][1], gridsize_x),
                      np.linspace(params['ylims'][0], params['ylims'][1], gridsize_y))

    dist_mat = np.sqrt(x_crds**2 + y_crds**2)
    #Find cutoff points for ring model
    maxdist = np.max(dist_mat)
    mindist = np.min(dist_mat)
    adjust = 0.10*gridsize_x    #center ring is 10% of total distance
    cutoff = np.linspace(mindist+adjust, maxdist, 4)
    print("New Ring boundaries = ")
    print(cutoff)


    #params['ring_mobi'] = 25*[0.6e3, 1.8e3, 3.2e3, 9.9e3]
    params['ring_mobi'] = cutoff
    sar_fullgrid[dist_mat <= params['ring_mobi'][0]] += 0.25 # center ring (95%)
    sar_fullgrid[dist_mat <= params['ring_mobi'][1]] += 0.25 # middle ring (70%)
    sar_fullgrid[dist_mat <= params['ring_mobi'][2]] += 0.25 # outter ring (45%)
    sar_fullgrid[dist_mat <= params['ring_mobi'][3]] += 0.20 # containment ring (20%)
    sar_fullgrid = ndimage.gaussian_filter(sar_fullgrid, sigma=64) # to make betta heatmap
    # normalize heatmap to sum to 1
    sar_fullgrid = sar_fullgrid - np.min(sar_fullgrid)
    sar_fullgrid = sar_fullgrid/np.max(sar_fullgrid)

    #For CPO 'penalty' will be 1-ring_prob for a grid.
    #Show heatmap and save numpy array
    saveprobs = False
    if(saveprobs):
        plt.imshow(sar_fullgrid)
        plt.title('Selected heatmap')
        plt.show()
        np.save('./deeprl_data/ring_prob.npy',sar_fullgrid)

    stime = time.time()
    planner = planning.Planning(params, on_terrain=mc.terrain, mode='TOTALDIST') # also calls terrain builder...
    rgp = robotgp.RobotGP(mc, planner, _stime = stime, parameters = params)
    rgp.collect_trainData() # sets out paths for each robot

    rgp.optimize_risk_cost_grad(_show_detail=True)


    #Show optimized waypoints on matrix
    sar_paths = np.zeros((gridsize_x,gridsize_y),dtype=float)

    robot_paths = []
    start_idx = 0
    robot_idx = 0
    for path_len in rgp.robot_path_len:
        end_idx = start_idx + path_len
        rpath = rgp.min_risk_paths[start_idx:end_idx]
        rpath = np.int32(np.rint(rpath))
        print('Path shape')
        print(np.shape(rpath))
        waypoints = np.zeros((np.shape(rpath)[0],2),dtype=np.int32())
        #Coordinates are from (-xlim, xlim) and (-ylim, ylim). Transform so 0,0 is corner like grid
        for i in range(0,np.shape(rpath)[0]):
            x = rpath[i][0]
            y = rpath[i][1]
            sar_paths[x+599][y+599] += 1
            waypoints[i][0] = x + params['xlims'][1]
            waypoints[i][1] = y + params['ylims'][1]
        #Plot path to make sure we're right
        plt.scatter(waypoints[:,0], waypoints[:,1])
        #save numpy array
        title = './deeprl_data/robot' + str(robot_idx) + '_waypoints.npy'
        np.save(title,waypoints)
        #Indices
        start_idx = end_idx
        robot_idx += 1
    #Show waypoints (should be overhead view)
    plt.show()


    plt.imshow(sar_paths)
    plt.title('Optimized Path Waypoints')
    plt.show()

def main():

    create_data()
    #Load robot paths from file


    #Make Gym Environment


    return











if __name__ == "__main__":
    main()
